# Requirements
This system requires access to a Hadoop cluster as well as large amounts of storage (At least 700GB)
You will probably also need a good understanding of how Hadoop MapReduce works and how to use HDFS, learn this before doing anything

# Building
Building the framework is done via maven:
"mvn clean"
"mvn install"
"mvn assembly:assembly -DdescriptorId=jar-with-dependencies"
Your built Jar file will be located in target/

# Step-by-step guide
This is a rough step-by-step guide to generating runs using this framework.
Explaining every single class in this framework would take a whole book, so consider this a rough guide to help you get started
You will need to understand source code to properly use this framework, so don't be afraid to look at the code

## Download data
Download the 2014 filtered StreamCorpus from http://s3.amazonaws.com/aws-publicdatasets/trec/kba/kba-streamcorpus-2014-v0_3_0-kba-filtered/index.html
Store this on HDFS so that we can access it from Hadoop

Download truth data from
http://trec-kba.org/data/trec-kba-2014-10-15-truth-data.7980e9a4c332d38d71105bcc94f2b593.tar.gz
Store this on your local computer and unzip

## Extract mentions
Run (On Hadoop) class no.ntnu.trec.kba.bin.ExtractMentions -c [Path to StreamCorpus] -nv [Path no low recall name variants] -o [Path to mentions]
Low recall name variants are found in the resources/ folder of the thesis delivery

## Filter StreamCorpus
Before you process you need to filter out stream-ids in the file at [Path to mentions] you just generated
This can be done with some basic python scripting, iterate over the [Path to mentions] file of the last stage and remove the entity part from each document-entity pair
So each line [doc_id]\t[ent_id] should become just [doc_id]
Output this to a new file [Path to stream ids] and use this file as input -s in the next command

Run (On Hadoop) class no.ntnu.trec.kba.bin.ExtractDocuments -c [Path to StreamCorpus] -nv [Path no low recall name variants] -s [Path to stream ids file] -o [filtered collection]

Now you have a smaller StreamCorpus collection contained in 25 SequenceFiles
To clean documents of whitespace and combine the 25 SequenceFiles
Run (On Hadoop) class no.ntnu.trec.kba.bin.CleanDocuments -c [filtered collection] -o [cleaned collection]

You now have the final filtered collection located at [cleaned collection]

## Generate mentions MapFile
For quicker lookup of mentions during feature extraction you need to convert the mentions into a Hadoop MapFile
Run (On Hadoop) class no.ntnu.trec.kba.bin.MentionsToHadoopMapfile -i [Path to mentions] -o [Path to mapfile]

## Perform feature extraction
You can now extract features using either the feature extractor on Hadoop (FeatureExtractorHadoop) or the local version (FeatureExtractor)

Run (On Hadoop) class no.ntnu.trec.kba.bin.FeatureExtractorHadoop -i [cleaned collection] -m [Path to mapfile] -f [name of feature extractor] -o [Path to feature vector]
or
Run (On local) class no.ntnu.trec.kba.bin.FeatureExtractor -i [cleaned collection] -m [Path to mapfile] -f [name of feature extractor] -o [Path to feature vector]

Each feature extractor also takes arguments in the form -F* which are extractor specific arguments such as paths to resources they need to run (entity profiles, temporal data, etc.)

There are too many details about each feature extractors to explain in a step-by-step guide, you should read the source code to understand how they work before using them.

## Combine feature output
Once you have extracted features from several different extractors you will want to combine them so you can use them at the same time

Run (On Hadoop) class no.ntnu.trec.kba.bin.FeatureOutputCombiner -i [Path to feature vector files] -o [Path to combined vector] -e [Optional regex to remove unwanted features]

In order to combine feature vectors from differnt folders use wildcard matchers in the -i argument
For example "*ACTIVE/part*" will combine features from all folders in the home directory that end with ACTIVE

## Run ranker
Now you can finally perform ranking on the feature vectors

### Gradient Boosted Trees
GBT rankers are implemented in Python meaning they require a bit more work to use

Run (On Hadoop) class no.ntnu.trec.kba.utils.classification.FeatureOutput2LibSVM -i [Path to combined vector] -a [Path to training annotations] -o [Path to libSVM output]
python src/main/python/vital_filtering.py --train [Path to libSVM output]/data.train --test [Path to libSVM output]/data.test --method ltr (add --dependent for entity-dependent ranker)

### Random Forests
Run (On Hadoop) class no.ntnu.trec.kba.utils.classification.RankLibRanker -i [Path to combined vector] -a [Path to training annotations] -o .

Your runfiles is now at "ntnu-default.gz"

This is the final output, you can run this through the offical scorer to evaluate the run
