package no.ntnu.trec.kba;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FSDataInputStream;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapreduce.InputSplit;
import org.apache.hadoop.mapreduce.RecordReader;
import org.apache.hadoop.mapreduce.TaskAttemptContext;
import org.apache.hadoop.mapreduce.lib.input.FileSplit;
import org.apache.thrift.TException;
import org.apache.thrift.protocol.TBinaryProtocol;
import org.apache.thrift.protocol.TProtocol;
import org.apache.thrift.transport.TIOStreamTransport;
import org.apache.thrift.transport.TTransportException;
import org.mortbay.util.IO;
import org.apache.commons.compress.compressors.xz.XZCompressorInputStream;

import java.io.BufferedInputStream;
import java.io.IOException;

/**
 * Created by NT.
 * User: takhirov
 * Date: 9/8/12
 * Time: 11:24 PM
 */
public class ThriftRecordReader extends RecordReader<Text, StreamItemWritable> {

	private FSDataInputStream in;
	private TProtocol tp;
	private long start;
	private long length;
	private long position;
	private Text key = new Text();
	private StreamItemWritable value = new StreamItemWritable();
	private FileSplit fileSplit;
	private Configuration conf;

	public ThriftRecordReader(FileSplit fileSplit, Configuration conf) throws IOException {
		this.fileSplit = fileSplit;
		this.conf = conf;
	}

	@Override public void close() throws IOException {
		IO.close(in);}

	/** Returns our progress within the split, as a float between 0 and 1. */
	@Override
	public float getProgress() {

		if (length == 0)
			return 0.0f;

		return Math.min(1.0f, position / (float) length);

	}

	/** Boilerplate initialization code for file input streams. */
	@Override
	public void initialize(InputSplit split, TaskAttemptContext context) throws IOException, InterruptedException {

		conf = context.getConfiguration();
		fileSplit = (FileSplit) split;
		start = fileSplit.getStart();
		length = fileSplit.getLength();
		position = 0;

		Path path = fileSplit.getPath();
		FileSystem fs = path.getFileSystem(conf);
		in = fs.open(path);
		XZCompressorInputStream xzInputStream = new XZCompressorInputStream(in);
		BufferedInputStream bi = new BufferedInputStream(xzInputStream);

		tp = new TBinaryProtocol.Factory().getProtocol(new TIOStreamTransport(bi));

	}

	@Override
	/**
	 * parse the next key value, update position and return true
	 */
	public boolean nextKeyValue() throws IOException, InterruptedException {

		// key
		// key.set(fileSplit.getPath().getName().toString());
		key.set(fileSplit.getPath().toString());

		// value
		if (in.available() > 0) { // && position - start < length) {
			try {
				value.read(tp);
				position = length - in.available() - start;
			} catch (TTransportException te) {
				// Deal with the EOF exception bug
				if (te.getType() == TTransportException.END_OF_FILE
						|| te.getCause() instanceof java.io.EOFException) {
					System.out.println("Failed map/file: " + fileSplit.getPath().toString());
					return false;
				} 
			} catch (Exception e) {
				throw new IOException(e);
			}
		} else {
			return false;
		} 
		return true;
	}

	@Override
	public Text getCurrentKey() throws IOException, InterruptedException {
		return key;
	}

	@Override
	public StreamItemWritable getCurrentValue() throws IOException, InterruptedException {
		return value;
	}
}
