package no.ntnu.trec.kba.features;

import no.ntnu.trec.kba.StreamDocument;
import no.ntnu.trec.kba.TopicEntity;
import no.ntnu.utils.*;
import org.apache.hadoop.conf.Configuration;
import org.apache.lucene.analysis.Analyzer;
import streamcorpus.Sentence;

import java.io.IOException;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;


/**
 * Extract clustering features
 *
 * @author Christian Barth Roligheten <chrbarrol@gmail.com>
 */
public class Clustering extends FeatureGenerator {

	private DateFormat inputDateFormat = new SimpleDateFormat("yyyy-MM-dd-HH");


	private HashMap<String, SinglePassClusters> entityClusters;

	private long getTimeDifferenceHours(String timestamp1, String timestamp2) {
		try {
			Date time1 = inputDateFormat.parse(timestamp1);
			Date time2 = inputDateFormat.parse(timestamp2);
			long diff = time1.getTime() - time2.getTime();
			return TimeUnit.MILLISECONDS.toSeconds(diff)/3600;

		} catch (ParseException e) {
			e.printStackTrace();
			return 0;
		}
	}

	public Clustering(Configuration config) throws IOException {
		super(config);
		entityClusters = new HashMap<String, SinglePassClusters>();
	}

	@Override
	public String getFeatureVector(StreamDocument doc,
			TopicEntity entity) {
		List<Sentence> sentences = NlpHelper.getMentioningSentences(doc.getSentences(), entity.getNameVariantsPattern());

		Map<String, Integer> docSet = NlpHelper.getFreqMapFromSentences(sentences);

		SinglePassClusters entityCluster = entityClusters.get(entity.getName());
		if (entityCluster == null) {
			entityCluster = new SinglePassClusters(0.9f);
			entityClusters.put(entity.getName(), entityCluster);
		}

		SinglePassClusters.Cluster cluster = entityCluster.addDocumentToCluster(docSet, doc.get(StreamDocument.FIELD_DIR_NAME));
		float hourAfterFirst = getTimeDifferenceHours(doc.get(StreamDocument.FIELD_DIR_NAME), cluster.getTimestamp());
		return "_cl:" + hourAfterFirst;
	}

}
