package no.ntnu.trec.kba.features;

import java.io.IOException;
import java.util.HashMap;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.mapreduce.Mapper;
import org.apache.lucene.analysis.Analyzer;

import no.ntnu.trec.kba.StreamDocument;
import no.ntnu.trec.kba.TopicEntity;
import no.ntnu.utils.LanguageDetector;
import no.ntnu.utils.TextAnalysis;

/**
 * Document features
 * 
 * @author Krisztian Balog <krisztian.balog@idi.ntnu.no>
 */
public class Document extends FeatureGenerator {

	private LanguageDetector ld;

	public Document(Configuration config) throws IOException {
		super(config);
		ld = new LanguageDetector();
	}

	@Override
	public String getFeatureVector(StreamDocument doc,
			TopicEntity entity) {

		// field lengths
		Analyzer analyzer = entity.getAnalyzer();
		String body = doc.get(StreamDocument.FIELD_BODY);
		String[] bodyTerms = TextAnalysis.analyze(analyzer, body);

		// source
		String source = doc.get("source");
		String src = "?";
		if (source.equals("news")) {
			src = "0";
		} else if (source.equals("social")) {
			src = "1";
		} else if (source.equals("linking")) {
			src = "2";
		} else if (source.equals("arxiv")) {
			src = "3";
		} else if (source.equals("CLASSIFIED")) {
			src = "4";
		} else if (source.equals("FORUM")) {
			src = "5";
		} else if (source.equals("MAINSTREAM_NEWS")) {
			src = "6";
		} else if (source.equals("MEMETRACKER")) {
			src = "7";
		} else if (source.equals("REVIEW")) {
			src = "8";
		} else if (source.equals("WEBLOG")) {
			src = "9";
		}

		// language detection
		String lang;

		if (ld.isEnglish(body))
			lang = "1";
		else
			lang = "0";
	
		String fv = "LB:" + bodyTerms.length + "\t"
				+ "SRC:" + src + "\t"
				+ "ENG:" + lang;

		return fv;
	}

}
