package no.ntnu.trec.kba.features;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.regex.Pattern;

import no.ntnu.trec.kba.StreamDocument;
import no.ntnu.trec.kba.TopicEntity;
import no.ntnu.utils.MentionTextHelper;
import no.ntnu.utils.TextAnalysis;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.mapreduce.Mapper;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.document.Document;

/**
 * Extract entity mention related features from document
 * 
 * @author Krisztian Balog <krisztian.balog@idi.ntnu.no>
 */
public class EntityMentions extends FeatureGenerator {

	public EntityMentions(Configuration config) throws IOException {
		super(config);
	}

	@Override
	public String getFeatureVector(StreamDocument doc, TopicEntity entity) {

		// entity name variants
		Pattern nvPattern = entity.getNameVariantsPattern();

		String text = doc.get(StreamDocument.FIELD_BODY);

		List<Integer> idxBody = MentionTextHelper.getMentionPositions(text, nvPattern);

		int fpos = 0;
		int lpos = 0;
		if (idxBody.size() > 0) {
			fpos = Collections.min(idxBody);
			lpos = Collections.max(idxBody);
		}
		// spread is 0 if there is only a single entity mention
		int spr = fpos > 0 ? lpos - fpos : 0;
		double fposNorm = text.length() > 0 ? (double) fpos / text.length() : 0.0;
		double lposNorm = text.length() > 0 ? (double) lpos / text.length() : 0.0;
		double sprNorm = text.length() > 0 ? (double) spr / text.length() : 0.0;
		return "MenBo:" + idxBody.size() + "\t" // N(D_body, E)
				+ "FPos:" + fpos + "\t" // FPOS(D, E)
				+ "LPos:" + lpos + "\t" // LPOS(D, E)
				+ "Spr:" + spr + "\t" // SPR(D, E)
				+ "FPosN:" + fposNorm + "\t" // FPOS_norm(D, E)
				+ "LPosN:" + lposNorm + "\t" // LPOS_norm(D, E)
				+ "SprN:" + sprNorm; // SPR_norm(D, E)
	}

}
