package no.ntnu.trec.kba.features;

import java.io.IOException;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.mapreduce.Mapper;

/**
 * Feature factory
 * 
 * @author Krisztian Balog <krisztian.balog@idi.ntnu.no>
 */
public class FeatureFactory {

	public static FeatureGenerator getFeatureGenerator(String featureSet, Configuration config) throws IOException {
		FeatureGenerator fg = null;

		if (featureSet.equals("EntityMentions")) {
			fg = new EntityMentions(config);
		} else if (featureSet.equals("Document")) {
			fg = new Document(config);
		} else if (featureSet.equals("ProfileSimilarity")) {
			fg = new ProfileSimilarity(config);
		} else if (featureSet.equals("Temporal")) {
			fg = new Temporal(config);
		} else if (featureSet.equals("KurtosisTrend")) {
			fg = new KurtosisTrend(config);
		} else if (featureSet.equals("Verb")) {
			try {
				fg = new Verbs(config);
			} catch (IOException e) {
				e.printStackTrace();
			}
		} else if (featureSet.equals("VerbEmbeddings")) {
			try {
				fg = new VerbsEmbedding(config);
			} catch (IOException e) {
				e.printStackTrace();
			}
		} else if (featureSet.equals("Cluster")) {
			try {
				fg = new Clustering(config);
			} catch (IOException e) {
				e.printStackTrace();
			}
		} else if (featureSet.equals("SimilaritySemantic")) {
			try {
				fg = new ProfileSimilaritySemantic(config);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}

		return fg;
	}
}
