package no.ntnu.trec.kba.features;

import no.ntnu.trec.kba.StreamDocument;
import no.ntnu.trec.kba.TemporalEntityData;
import no.ntnu.trec.kba.TopicEntity;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.mapreduce.Mapper;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;

/**
 * Extract Kurtosis and Trend features
 *
 * @author Christian Barth Roligheten <chrbarrol@gmail.com>
 */
public class KurtosisTrend extends FeatureGenerator {

	private final static int[] HOURS = { 12, 24, 48, 96, 192 };

	public KurtosisTrend(Configuration config) throws IOException {
		super(config);
	}

	@Override
	public String getFeatureVector(StreamDocument doc, TopicEntity entity) throws IOException {

		TemporalEntityData ted = entity.getTemporalData();

		String f = "";

		String timestamp = doc.get(StreamDocument.FIELD_DIR_NAME);

		boolean first = true;
		for (int h : HOURS) {
			if (first) {
				first = false;
			} else {
				f += "\t";
			}
			f += "TRND" + h + ":" + ted.getTrend(timestamp, h)
					+ "\tKURT" + h  + ":" + ted.getStreamKurtosis(timestamp, h);
		}

		return f;
	}
}
