package no.ntnu.trec.kba.features;

import java.io.IOException;
import java.util.HashMap;

import no.ntnu.trec.kba.StreamDocument;
import no.ntnu.trec.kba.TopicEntity;
import no.ntnu.utils.HadoopArguments;
import no.ntnu.utils.TermFreq;
import no.ntnu.utils.TextAnalysis;
import no.ntnu.utils.TextSimilarity;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.mapreduce.Mapper;
import org.apache.lucene.document.Document;

/**
 * Extract profile similarity features
 *
 * @author Christian Barth Roligheten <chrbarrol@gmail.com>
 */
public class ProfileSimilarity extends FeatureGenerator {

	private TermFreq termFreq;

	public ProfileSimilarity(Configuration config) throws IOException {
		super(config);
		termFreq = new TermFreq(HadoopArguments.getArgFromConfig(config,"Ff"),
				FileSystem.get(config),
				"body_cleansed");
	}

	@Override
	public String getFeatureVector(StreamDocument doc, TopicEntity entity) {

		String[] docText = TextAnalysis.analyze(
				TextAnalysis.getStandardAnalyzer(),
				doc.get(StreamDocument.FIELD_BODY));
		String[] entityText = entity.getTextAnalyzed();

		return "JacProf:" + TextSimilarity.jaccardSimilarity(docText, entityText)
				+ "\tCosProf:" + TextSimilarity.cosineSimilarity(docText, entityText, termFreq) +
				"\tKlDProf:" + TextSimilarity.klDivergence(docText, entityText, termFreq); // KL-Div
	}

}
