package no.ntnu.trec.kba.features;

import com.crtomirmajer.wmd4j.WordMovers;
import no.ntnu.trec.kba.StreamDocument;
import no.ntnu.trec.kba.TopicEntity;
import no.ntnu.utils.*;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FileSystem;
import org.deeplearning4j.models.embeddings.loader.WordVectorSerializer;
import org.deeplearning4j.models.embeddings.wordvectors.WordVectors;
import org.nd4j.linalg.api.ndarray.INDArray;
import streamcorpus.Sentence;


import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Extract semantic profile similarity features
 *
 * @author Christian Barth Roligheten <chrbarrol@gmail.com>
 */
public class ProfileSimilaritySemantic extends FeatureGenerator {
	private WordVectors wordVectors;

	private double cosineDistance(INDArray a1, INDArray a2) {
		double a1sq = 0;
		double a2sq = 0;
		double a1a2 = 0;
		for (int i = 0; i < a1.columns(); i++) {
			double a1i = a1.getDouble(i);
			double a2i = a2.getDouble(i);

			a1a2 += a1i*a2i;
			a1sq += a1i*a1i;
			a2sq += a2i*a2i;
		}

		return a1a2/(a1sq+a2sq);
	}

	public ProfileSimilaritySemantic(Configuration config) throws IOException {
		super(config);

		wordVectors = WordVectorSerializer.loadTxtVectors(
				new File(HadoopArguments.getArgFromConfig(config,"Fv")));
	}

	@Override
	public String getFeatureVector(StreamDocument doc, TopicEntity entity) {

		List<Sentence> sentences = NlpHelper.getMentioningSentences(doc.getSentences(), entity.getNameVariantsPattern());
		String tmp = "";
		List<String> tmpList = new ArrayList<String>();
		for (Sentence sent : sentences) {
			tmp += NlpHelper.sentenceToString(sent, true) + " ";
			tmpList.addAll(NlpHelper.sentenceToList(sent, true));
		}

		double wvAvg = 0;

		// Catch exception caused by 0 vector
		try {
			INDArray docAvg = wordVectors.getWordVectorsMean(tmpList);
			INDArray entAvg = wordVectors.getWordVectorsMean(Arrays.asList(entity.getTextAnalyzed()));
			wvAvg = cosineDistance(docAvg, entAvg);
		} catch (IllegalArgumentException e) {
			// LEave wvAvg 0 as default when err
		}

		// WMD dropped because of computation time
		double wmd = 0; //wordMover.distance(tmp, String.join(" ", entity.getTextAnalyzed()));

		return "WMDProf:" + wmd
				+ "\tAVGWVProf:" + wvAvg;
	}

}
