package no.ntnu.trec.kba.features;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.mapreduce.Mapper;
import org.apache.lucene.document.Document;

import no.ntnu.trec.kba.StreamDocument;
import no.ntnu.trec.kba.TemporalEntityData;
import no.ntnu.trec.kba.TopicEntity;

/**
 * Temporal features
 *
 * @author Krisztian Balog <krisztian.balog@idi.ntnu.no>
 */
public class Temporal extends FeatureGenerator {

	private final static int[] HOURS = { 2, 4, 8, 16, 32 };

	public Temporal(Configuration config) throws IOException {
		super(config);
	}

	@Override
	public String getFeatureVector(StreamDocument doc, TopicEntity entity) throws IOException {

		TemporalEntityData ted = entity.getTemporalData();

		String f = "SVAvg:" + new Double(ted.getAvgStreamVolume()).toString(); // SVavg

		String timestamp = doc.get(StreamDocument.FIELD_DIR_NAME);

		for (int h : HOURS) {
			f += "\tSV" + h + ":" + ted.getStreamVolume(timestamp, h) // SV
					+ "\tdSV" + h + ":" + ted.getStreamVolumeChange(timestamp, h) // dSV
					+ "\tbS" + h + ":" + ted.isBurstStream(timestamp, h); // SB
		}

		return f;
	}
}
