package no.ntnu.trec.kba.trainingset;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;

import org.apache.hadoop.io.Text;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.analysis.util.CharArraySet;
import org.apache.lucene.util.Version;

import no.ntnu.trec.kba.StreamDocument;
import no.ntnu.utils.EntityNameResolver;
import no.ntnu.utils.TextAnalysis;

public class TrainingDocumentFrequencyGenerator extends TrainingSetProcessor {
	HashMap<String, Integer> dfs, cfs;
	private int counter = 0;
	Analyzer analyzer;
	
	public TrainingDocumentFrequencyGenerator(String entitiesFile, String annotationsFile, String outputDir) throws IOException {
		super(entitiesFile, annotationsFile, outputDir);
		analyzer = new StandardAnalyzer(Version.LUCENE_42, CharArraySet.EMPTY_SET);
	
		dfs = new HashMap<String, Integer>();
		cfs = new HashMap<String, Integer>();

	}

	@Override
	void map(Text key, StreamDocument item) {
		String[] bodyTerms = TextAnalysis.analyze(analyzer, item.get(StreamDocument.FIELD_BODY));
		HashSet<String> usedTerms = new HashSet<String>();
		for (String term : bodyTerms) {
			if (cfs.containsKey(term)) {
				cfs.put(term, cfs.get(term)+1);
			} else {
				cfs.put(term, 1);
			}
			
			if (usedTerms.contains(term))
				continue;
			if (dfs.containsKey(term)) {
				dfs.put(term, dfs.get(term)+1);
			} else {
				dfs.put(term, 1);
			}
			usedTerms.add(term);
		}
		counter++;
	}

	@Override
	void after() throws IOException {
		Files.write(Paths.get(outputDir), new Integer(counter).toString().getBytes(), StandardOpenOption.CREATE);
		Files.write(Paths.get(outputDir), "\n".getBytes(), StandardOpenOption.APPEND);

		for (String term : dfs.keySet()) {
			int df = dfs.get(term);
			int cf = cfs.get(term);
			String out = term + "\t" + Integer.toString(df) + "\t" + Integer.toString(cf) + "\n";
			if (dfs.get(term) > 10) {
				Files.write(Paths.get(outputDir), out.getBytes(), StandardOpenOption.APPEND);
			}
		}
		System.out.println(dfs.get("a").toString());
		System.out.println(counter);
		
	}
	
	public static void main(String[] args) {
		try {
			TrainingDocumentFrequencyGenerator generator = new TrainingDocumentFrequencyGenerator(args[0], args[1], args[2]);
			generator.run(args[3]);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

}
