package no.ntnu.trec.kba.trainingset;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FileStatus;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.SequenceFile;
import org.apache.hadoop.io.SequenceFile.Reader;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapred.SequenceFileRecordReader;
import org.apache.lucene.store.Directory;

import no.ntnu.trec.kba.StreamDocument;
import no.ntnu.utils.EntityNameResolver;

public abstract class TrainingSetProcessor {
	protected HashSet<String> entities;
	protected HashMap<String, Integer> annotations;
	protected HashMap<String, ArrayList<String>> mentions;
	protected String outputDir;
	
	public TrainingSetProcessor(String entitiesFile, String annotationsFile, String outputDir) throws IOException {
		this.outputDir = outputDir;
		List<String> lines = Files.readAllLines(Paths.get(entitiesFile));
		
		entities = new HashSet<String>();
		for (String line : lines) {
			entities.add(line);
		}
		
		lines = Files.readAllLines(Paths.get(annotationsFile));
		
		annotations = new HashMap<String, Integer>();
		mentions = new HashMap<String, ArrayList<String>>();

		for (String line : lines) {
			if (line.startsWith("#")) {
				continue;
			}
			String[] split = line.split("\t");
			String id = split[2];
			String mention = split[3];
			String annot = split[5];
			
			if (mentions.containsKey(id)) {
				mentions.get(id).add(mention);
			} else {
				ArrayList<String> n = new ArrayList<String>();
				n.add(mention);
				mentions.put(id, n);
			}
			
			annotations.put(id+"\t"+mention, Integer.decode(annot));
			
			
			
		}
		
	}
	
	abstract void map(Text key, StreamDocument item);
	abstract void after() throws IOException;
	
	public void run(String path) throws IOException {
		Configuration conf = new Configuration();
		FileSystem fs = FileSystem.get(conf);
		
		for (FileStatus file : fs.listStatus(new Path(path))) {
			if (!file.getPath().getName().startsWith("part")) {
				continue;
			}
			SequenceFile.Reader reader = new SequenceFile.Reader(conf, Reader.file(file.getPath()));
			
			Text key = new Text();
			StreamDocument doc = new StreamDocument();
					
			while (reader.next(key, doc)) {
				map(key, doc);
			}
			reader.close();

		}	
		after();

	}
}
