package no.ntnu.utils;

import java.io.IOException;
import java.io.Reader;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.TokenStream;
import org.apache.lucene.analysis.core.LowerCaseFilter;
import org.apache.lucene.analysis.core.StopAnalyzer;
import org.apache.lucene.analysis.core.StopFilter;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.analysis.standard.StandardFilter;
import org.apache.lucene.analysis.standard.StandardTokenizer;
import org.apache.lucene.util.Version;

/**
 * A custom analyzer which basically does the same thing as a StandardAnalyzer
 * plus using i a custom EntityTokenFilter to remove char we don't need
 * 
 * @author Heri Ramampiaro <heri@ntnu.no>
 * 
 */

public class EntityAnalyzer extends Analyzer {

	private Version matchVersion = Version.LUCENE_40;
	private final Analyzer DEFAULT_BASE_ANALYZER = new StandardAnalyzer(
			matchVersion);
	private Analyzer baseAnalyzer;

	protected EntityAnalyzer(Version matchverion) {
		super();
		matchVersion = matchverion;
		baseAnalyzer = DEFAULT_BASE_ANALYZER;

	}

	protected EntityAnalyzer(Analyzer a, Version matchverion) {
		super();
		matchVersion = matchverion;
		baseAnalyzer = a;

	}

	@Override
	protected TokenStreamComponents createComponents(String field, Reader reader) {

		final StandardTokenizer src = new StandardTokenizer(matchVersion,
				reader);
		TokenStream result = new StandardFilter(matchVersion, src);
		result = new LowerCaseFilter(matchVersion, result); // Lower all cases
		result = new StopFilter(matchVersion, result,
				StopAnalyzer.ENGLISH_STOP_WORDS_SET); // Remove stopwords using
														// the standard list
		result = new EntityTokenFilter(result); // Use our own entity
												// tokenfilter to remove char we
												// don't need
		return new TokenStreamComponents(src, result) {
			@Override
			protected void setReader(final Reader reader) throws IOException {
				src.setMaxTokenLength(StandardAnalyzer.DEFAULT_MAX_TOKEN_LENGTH);
				super.setReader(reader);
			}
		};

	}

}
