package no.ntnu.utils;

import org.apache.hadoop.conf.Configuration;
import org.apache.log4j.Logger;

/**
 * Automatically passes command line arguments to Hadoop
 *
 * @author Christian Barth Roligheten <chrbarrol@gmail.com>
 */

public class HadoopArguments {
    private static final String CONF_PATH_PREFIX = "argv";

    private static final Logger log = Logger.getLogger(HadoopArguments.class);


    private static String getArgPath(String arg) {
        return CONF_PATH_PREFIX + "." + arg;
    }

    public static void addArgsToConfig(Configuration conf, String[] args) throws ArrayIndexOutOfBoundsException {
        for (int i = 0; i < args.length; ++i) {
            try {
                String arg = getArgPath(args[i].substring(1));
                String val = args[++i];
                conf.set(arg, val);
                log.info("Added arg " + arg + " with value " + val);
            } catch (ArrayIndexOutOfBoundsException e) {
                throw new ArrayIndexOutOfBoundsException("Parameter missing from " + args[i - 1]);
            }
        }
    }

    public static String getArgFromConfig(Configuration config, String argName) {
        return config.get(getArgPath(argName));
    }
}
