package no.ntnu.utils;

import java.text.DecimalFormat;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;


/**
 * Jaccard implementation.
 *
 * @author <a href="mailto:takhirov@idi.ntnu.no">Naimdjon Takhirov</a>
 */
public class Jaccard {

    /**
     * Calculates the jaccard score of the tokens in the two strings. The tokens are obtained by splitting the string with the whitespace.
     * This method performs calculation without taking case sensitivity into consideration.
     *
     * @param A the first string
     * @param B the second string
     * @return jaccard score between the two strings disregarding the case sensitivity.
     */
    public static double calculateScoreIgnoreCase(String A, String B) {
        Set<String> A_set = new HashSet<String>();

        for (String s : A.split(" "))
            A_set.add(s.toLowerCase());

        Set<String> B_set = new HashSet<String>();
        for (String s : B.split(" "))
            B_set.add(s.toLowerCase());
        return calculateScore(A_set, B_set);
    }


    /**
     * Calculates the jaccard score of the tokens in the two strings. The tokens are obtained by splitting the string with the whitespace.
     *
     * @param A the first string
     * @param B the second string
     * @return jaccard score between the two strings.
     */
    public static double calculateScore(String A, String B) {
        Set<String> A_set = new HashSet<String>(Arrays.asList(A.split(" ")));
        Set<String> B_set = new HashSet<String>(Arrays.asList(B.split(" ")));
        return calculateScore(A_set, B_set);
    }

    /**
     * Calculates the jaccard score of the two sets.
     *
     * @param A the first set
     * @param B the second set
     * @return jaccard score between the two sets.
     */
    public static double calculateScore(Set<String> A, Set<String> B) {
        Set<String> intersection = new HashSet<String>(A);
        intersection.retainAll(B);
        Set<String> union = new HashSet<String>(A);
        union.addAll(B);
        return (double) intersection.size() / (double) union.size();
    }

    public static void main(String[] args) {
        Set<String> a = new HashSet<String>();
        Set<String> b = new HashSet<String>();
        a.add("test");
        a.add("string1");
        b.add("string2");
        b.add("test");
        String format = new DecimalFormat("#0.00").format(calculateScore(a, b));
        if (!format.equals("0.33"))
            throw new IllegalStateException("Calculate jaccard score is not working!" + format);
    }
}