package no.ntnu.utils;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class MentionTextHelper {
    private static final Pattern eosPattern = Pattern.compile("[.!?]");

    private static final int MAX_CHARS_BACKWARDS = 500;
    private static final int MAX_CHARS_FORWARD = 1000;

    private static int getWindowStartIndex(String doc, int mentionIdx) {
        Matcher eosMatcher = eosPattern.matcher(doc);

        int curIdx = 0;
        while (eosMatcher.find(curIdx)) {
            if (eosMatcher.start() > mentionIdx) {
                return Math.max(mentionIdx-MAX_CHARS_BACKWARDS, curIdx);
            }
            curIdx = eosMatcher.end();
        }

        return Math.max(0, mentionIdx-MAX_CHARS_BACKWARDS);
    }

    public static Pattern getPatternFromNameVariants(List<String> nameVariants) {
        String exp = "(^|[^\\w])(";
        boolean first = true;
        for (String variant : nameVariants) {
            if (first) {
                first = false;
            } else  {
                exp += "|";
            }
            exp += Pattern.quote(variant.toLowerCase());
        }
        exp += ")([^\\w]|$)";

        return Pattern.compile(exp);
    }

    private static int getWindowEndIndex(String doc, int mentionIdx) {
        Matcher eosMatcher = eosPattern.matcher(doc);

        if (eosMatcher.find(mentionIdx)) {
            return Math.min(eosMatcher.end(), mentionIdx+MAX_CHARS_FORWARD);
        }

        return Math.min(doc.length(), mentionIdx+MAX_CHARS_FORWARD);
    }

    public static boolean hasMoreThanOneMention(String doc, Pattern namePattern) {
        String docLower = doc.toLowerCase();
        Matcher matcher = namePattern.matcher(docLower);

        return matcher.find();
    }

    public static List<Integer> getMentionPositions(String doc, Pattern namePattern) {
        String docLower = doc.toLowerCase();
        Matcher matcher = namePattern.matcher(docLower);


        ArrayList<Integer> positions = new ArrayList<Integer>();
        while (matcher.find()) {
            positions.add(matcher.start());
        }

        return positions;
    }

    public static String combineMentionSentences(List<String> sentences) {
        StringBuilder builder = new StringBuilder();
        for (int idx = 0; idx < sentences.size(); idx++) {
            builder.append(sentences.get(idx));
            builder.append(". ");
        }

        return builder.toString();
    }

    public static List<String> getMentionWindows(String doc, Pattern namePattern) {
        String docLower = doc.toLowerCase();
        Matcher matcher = namePattern.matcher(docLower);

        ArrayList<String> out = new ArrayList<String>();
        int pos = 0;
        while (matcher.find(pos)) {
            int loc = matcher.start();
            int start = getWindowStartIndex(doc, loc);
            int end = getWindowEndIndex(doc, loc+matcher.group().length());

            String window = doc.substring(start, end);
            pos = end;
            out.add(window);
        }

        return out;
    }
}
