package no.ntnu.utils;

import edu.stanford.nlp.process.Morphology;
import edu.stanford.nlp.simple.Document;
import net.sf.extjwnl.JWNLException;
import net.sf.extjwnl.data.*;
import net.sf.extjwnl.data.list.PointerTargetNode;
import net.sf.extjwnl.data.list.PointerTargetNodeList;
import net.sf.extjwnl.data.list.PointerTargetTree;
import net.sf.extjwnl.data.list.PointerTargetTreeNode;
import net.sf.extjwnl.data.relationship.RelationshipFinder;
import net.sf.extjwnl.data.relationship.RelationshipList;
import net.sf.extjwnl.dictionary.Dictionary;
import no.ntnu.trec.kba.bin.GenerateKBARunFile;
import org.apache.log4j.Logger;
import streamcorpus.Sentence;

import java.util.*;

public class WordnetHelper {
    private static Dictionary wordnet = null;

    private static final Logger log = Logger.getLogger(no.ntnu.trec.kba.bin.VocabularyExtractor.class);


    public static Dictionary getWordnet() {
        if (wordnet == null) {
            try {
                wordnet = Dictionary.getDefaultResourceInstance();
            } catch (JWNLException e) {
                System.out.println("Failed to get Wordnet: " + e.getMessage());
            }
        }

        return wordnet;
    }

    private static HashSet<String>  extractAllSynsetOffsetsFromTree(PointerTargetTree tree) {
        // Walk the hypernym tree depth-first to extract hypernym synset offsets
        Stack<PointerTargetTreeNode> stack = new Stack<PointerTargetTreeNode>();
        if (tree.getRootNode() != null) {
            stack.add(tree.getRootNode());
        }

        HashSet<String> synsetOffsets = new HashSet<String>();
        HashSet<String> synsetWords = new HashSet<String>();

        while (!stack.isEmpty()) {
            PointerTargetTreeNode node = stack.pop();

            synsetOffsets.add(Long.toString(node.getSynset().getOffset()));


            if (node.getChildTreeList() == null || node.getChildTreeList().size() == 0) {
                synsetWords.add(Long.toString(node.getSynset().getOffset()));
                continue;
            }

            // Loop prevention
            for (PointerTargetTreeNode candidate : node.getChildTreeList()) {
                if (!synsetOffsets.contains(Long.toString(candidate.getSynset().getOffset()))) {
                    stack.add(candidate);
                }
            }
        }

        return synsetWords;
    }

    public static HashSet<String> getVerbHypernyms(String verb, String verbTag) {
        Dictionary wordnet = getWordnet();


        HashSet<String> hypernymOffsets = new HashSet<String>();
        try {
            verb = Morphology.lemmaStatic(verb, verbTag);
            IndexWord wordnetVerb = wordnet.getIndexWord(POS.VERB, verb);

            if (wordnetVerb == null) {
                return hypernymOffsets;
            }


            Synset mostFrequentSynset = getMostFrequentSense(wordnetVerb.getSenses());

            if (mostFrequentSynset == null) {
                return hypernymOffsets;
            }



            PointerTargetTree hypernymTree = PointerUtils.getHypernymTree(mostFrequentSynset, 5);

            hypernymOffsets.addAll(extractAllSynsetOffsetsFromTree(hypernymTree));


        } catch (JWNLException e) {
            log.error("Failed to get Wordnet: " + e.getMessage());
        }


        return hypernymOffsets;
    }

    private static Synset getMostFrequentSense(List<Synset> synsets) {
        if (synsets.size() > 0) {
            return synsets.get(0);
        }

        return null;
    }
}
