package no.ntnu.wikipedia;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.apache.avro.data.Json;
import org.jsoup.HttpStatusException;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;

import com.google.common.base.Strings;

public class ExtractExternalProfile {
	public static String getWaybackUrl(String url) throws IOException {
		URL apiUrl =  new URL("http://archive.org/wayback/available?url=" + url + "&timestamp=20120101");
		
		BufferedReader apiReader = new BufferedReader(new InputStreamReader(apiUrl.openStream()));
		
		char[] arr = new char[2048];
		apiReader.read(arr);
		String apiResponse = new String(arr);
		int startUrl = apiResponse.indexOf("http://web.archive.org/");
		if (startUrl == -1)
			return null;
		int stopUrl = apiResponse.indexOf("\"", startUrl);
		
		int startTime = apiResponse.indexOf("\"timestamp\": \"");
		startTime = apiResponse.indexOf("\"timestamp\": \"", startTime+1);
		
		String timestamp = apiResponse.substring(startTime+14, startTime+14+8);
		
		int a = timestamp.compareTo("20120101");
		if (timestamp.compareTo("20120101") > 0) {
			return null;
		}
		
		return apiResponse.substring(startUrl, stopUrl);
	}
	
	public static String getCitationContent(String url) throws IOException {
		Document doc =  Jsoup.connect(url).timeout(20000).get();
		return doc.text();
	}
	
	
	public static void run(String inputFile, String outputDir) throws IOException {
		Path in = Paths.get(inputFile);
		HashMap<String, String> profiles = new HashMap<String, String>();
		
		List<String> lines = Files.readAllLines(in);
		for (String line : lines) {
			String[] split = line.split("\t");
			
			try {
				String url = getWaybackUrl(split[1]);
				if (url == null)
					continue;
				String content = getCitationContent(url);
				String prof = profiles.getOrDefault(split[0], "");
				
				profiles.put(split[0], prof + " " + content);
				
			} catch (IOException e1) {
				continue;
			}
		}
		
		for (String name : profiles.keySet()) {
			Files.write(Paths.get(outputDir + "/" + name), profiles.get(name).getBytes(), StandardOpenOption.CREATE);
		}

	}
	
	
	public static void main(String[] args) throws IOException {
		String wikipagesDir = args[0];
		String outputDir = args[1];
		
		ExtractExternalProfile.run(wikipagesDir, outputDir);
	}
}
