import output
import numpy
import xgboost as xgb
from xgboostextension import XGBRanker
from math import ceil
import dataset

def read_groups(f):
    groups = []
    for line in f:
        groups.append(int(line.strip()))
    return groups

def normalize_scores(y):
    ma = numpy.amax(y)
    mi = numpy.amin(y)

    new = None
    if ma == mi:
        new = [1 for x in y]
    else:
        new = [1+int(ceil((x-mi)/(ma-mi)*999)) for x in y]

    return new

def rescale_ranks(ranks):
    res = []
    for rank in ranks:
        if rank > 2:
            rank = 1
        else:
            rank = 0
        res.append(rank)
    return res

def predict_single(train, train_groups, test, test_groups, ranker=None, print_importance=False):
    if ranker is None:
        ranker = XGBRanker()
        ranker.fit(train, None, train_groups)

    if print_importance:
        print('Feature importance:')
        for feat in ranker.feature_importances_:
            print(feat)
    y_pred = ranker.predict(test, test_groups)
    return y_pred

def predict_entity_dependent(train, train_groups, test, test_groups):
    if len(train_groups) != len(test_groups):
        print('WARNING: Group counts not equal in test/train sets. Resulting predictions will be invalid!')


    global_ranker = XGBRanker()
    global_ranker.fit(train.slice(range(train.num_row())), None, train_groups)

    cur_train_idx = 0
    cur_test_idx = 0
    y_pred = []
    for group_idx in range(len(train_groups)):
        train_group_size = train_groups[group_idx]
        test_group_size = test_groups[group_idx]

        ent_test = test.slice(range(cur_test_idx, cur_test_idx+test_group_size))

        ent_y = None
        if train_group_size < 50:
            ent_y = predict_single(train, train_groups, ent_test, [test_group_size], global_ranker)
        else:
            ent_train = train.slice(range(cur_train_idx, cur_train_idx+train_group_size))
            ent_y = predict_single(ent_train, [train_group_size], ent_test, [test_group_size])


        y_pred.extend(ent_y)

        cur_train_idx += train_group_size
        cur_test_idx += test_group_size

    return y_pred

def run(args):
    train_set = xgb.DMatrix(args.train)
    test_set = xgb.DMatrix(args.test)

    #train_set.set_label(rescale_ranks(train_set.get_label()))

    train_groups = None
    test_groups = None
    ids = None
    with open(args.train + '.groups') as f:
        train_groups = read_groups(f)
    with open(args.test + '.groups') as f:
        test_groups = read_groups(f)
    with open(args.test + '.ids') as f:
        ids = output.load_ids(f)

    y_pred = None
    if not args.dependent:
        y_pred = predict_single(train_set, train_groups, test_set, test_groups, None, args.print_importance)

    else:
        y_pred = predict_entity_dependent(train_set, train_groups, test_set, test_groups)
    y_pred = normalize_scores(y_pred)

    with open(args.runname, 'a+') as f:
        output.write_runfile_header(f)
        for idx in range(len(y_pred)):
            conf = y_pred[idx]
            doc_id = ids[idx]
            output.write_runfile_judgement(doc_id, conf, 2, f)

